#include "Connection.h"
#include <iostream>
#include <string>
#include "NetComm.h"
#include "CommandProcessor.h"

using namespace std;

Connection::Connection(SOCKET clientSocket) : mClientSocket(clientSocket)
{
}

Connection::~Connection()
{
	shutdown(mClientSocket, SD_SEND);
	closesocket(mClientSocket);
}

void Connection::AsyncStart()
{
	if (!QueueUserWorkItem(s_ThreadProc, this, 0))
	{
		wcerr << L"QueueUserWorkItem error: 0x" << hex << GetLastError() << endl;
	}
}

void Connection::SendString(const wchar_t *text)
{
	size_t cbToSend = sizeof(text[0]) * (wcslen(text) + 1);
	size_t cbTotalSent = 0;
	size_t cbSent = 0;

	while (cbTotalSent < cbToSend && SOCKET_ERROR !=
		(cbSent = send(mClientSocket, 
		reinterpret_cast<const char*>(text),
		cbToSend - cbTotalSent, 0)))
	{
		cbTotalSent += cbSent;
	}
	if (cbSent == SOCKET_ERROR)
	{
		wcerr << L"send error: 0x" << hex << WSAGetLastError() << endl;
	}
}

DWORD Connection::Run()
{
	static const size_t sMaxMessageSize = 32 * 1024; // 32 K of characters should be large enough for a message.
	wchar_t buffer[20];
	size_t cbBuffer = 0;
	wstring str = L"";

	while ((cbBuffer = recv(mClientSocket, 
		reinterpret_cast<char*>(buffer), sizeof(buffer) - 2, 0)) != 0 && 
		cbBuffer != SOCKET_ERROR)
	{
		const size_t cchBuffer = cbBuffer / sizeof(buffer[0]);
		for (size_t strIdx = 0; strIdx < cchBuffer; ++strIdx)
		{
			if (buffer[strIdx] == L'\0' || buffer[strIdx] == L'|')
			{
				CommandProcessor::ProcessInput(str.c_str());
				str = L"";
			}
			else
			{
				str += buffer[strIdx];
			}

			if (str.length() > sMaxMessageSize)
			{
				wcerr << L"Message size too large." << endl;
				break;
			}
		}
	}
	if (cbBuffer == SOCKET_ERROR)
	{
		cbBuffer = WSAGetLastError();
		wcerr << L"send error: 0x" << hex << cbBuffer << endl;
	}

	return cbBuffer;
}

DWORD WINAPI Connection::s_ThreadProc(LPVOID lpParameter)
{
	Connection *conn = reinterpret_cast<Connection*>(lpParameter);

	DWORD dwResult = conn->Run();
	delete conn;

	return dwResult;
}
