#include "stdhdrs.h"
#include "NetComm.h"
#include "Connection.h"
#include <iostream>

using namespace std;

void NetComm::ListenForConnections(const wchar_t *port)
{
	InitNetwork();

	ADDRINFOW *result = NULL, hints;
	ZeroMemory(&hints, sizeof(hints));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;
	hints.ai_flags = AI_PASSIVE;

	// Resolve the local address and port to be used by the server
	if (!GetAddrInfoW(NULL, port, &hints, &result))
	{
		SOCKET listenSocket = socket(result->ai_family, result->ai_socktype, result->ai_protocol);
		if (listenSocket != INVALID_SOCKET)
		{
			if (!bind(listenSocket, result->ai_addr, (int)result->ai_addrlen))
			{
				if (!listen(listenSocket, SOMAXCONN))
				{
					SOCKET clientSocket = INVALID_SOCKET;
					while (true)
					{
						char sockAddrBuffer[1024] = { 0 };
						sockaddr *newClient = reinterpret_cast<sockaddr*>(sockAddrBuffer);
						int cbNewClient = sizeof(sockAddrBuffer);

						clientSocket = accept(listenSocket, newClient, &cbNewClient);
						if (clientSocket != INVALID_SOCKET)
						{
							if (IsAcceptableClient(newClient))
							{
								(new Connection(clientSocket))->AsyncStart();
							}
						}
						else
						{
							wcerr << L"accept error: 0x" << hex << WSAGetLastError() << endl;
							break;
						}
					}
				}
				else
				{
					wcerr << L"listen error: 0x" << hex << WSAGetLastError() << endl;
				}
			}
			else
			{
				wcerr << L"bind error: 0x" << hex << WSAGetLastError() << endl;
			}
		}
		else
		{
			wcerr << L"socket error: 0x" << hex << WSAGetLastError() << endl;
		}
	}
	else
	{
		wcerr << L"GetAddrInfoW error: 0x" << hex << WSAGetLastError() << endl;
	}

	FreeAddrInfoW(result);

	UninitNetwork();
}

Connection* NetComm::ConnectTo(const wchar_t *hostname, const wchar_t *port)
{
	ADDRINFOW *addr = NULL, *ptr = NULL, hints;
	Connection *conn = NULL;

	ZeroMemory(&hints, sizeof(hints));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_protocol = IPPROTO_TCP;

	if (!GetAddrInfoW(hostname, port, &hints, &addr))
	{
		ptr = addr;

		int result = SOCKET_ERROR;	
		do
		{
			SOCKET connectSocket = socket(ptr->ai_family, ptr->ai_socktype, ptr->ai_protocol);
			result = connect(connectSocket, ptr->ai_addr, (int)ptr->ai_addrlen);

			if (result == SOCKET_ERROR)
			{
				ptr = ptr->ai_next;
				closesocket(connectSocket);
			}
			else
			{
				conn = new Connection(connectSocket);
			}
		}
		while (result == SOCKET_ERROR && ptr != NULL);
	}
	else
	{
		wcerr << L"GetAddrInfoW error: 0x" << hex << WSAGetLastError() << endl;
	}

	return conn;
}

void NetComm::AddAcceptableClient(const wchar_t *clientName)
{
	mClientNameList.push_back(clientName);
}

bool NetComm::IsAcceptableClient(const sockaddr *clientAddress) const
{
	bool acceptable = false;
	if (clientAddress)
	{
		// Max size of domain name.
		wchar_t clientName[256];

		if (!GetNameInfoW(
			clientAddress, sizeof(*clientAddress),
			clientName, ARRAYSIZE(clientName),
			NULL, 0, 0))
		{
			for (list<wstring>::const_iterator itr = mClientNameList.begin(); 
				!acceptable && itr != mClientNameList.end(); ++itr)
			{
				acceptable = !wcscmp(clientName, itr->c_str());
			}

			if (!acceptable)
			{
				wcerr << L"Connection request from unacceptable client rejected: " << clientName << endl;
			}
		}
	}

	return acceptable;
}

int NetComm::InitNetwork()
{
	WSADATA wsaData;
	return WSAStartup(MAKEWORD(2,2), &wsaData);
}

void NetComm::UninitNetwork()
{
	WSACleanup();
}